; Multiplatform 32k programmable game boy cartridge firmware v1.01a
;
; Copyright (C) September 2006 Noel Lemouel (stellae) <noel.lemouel@stellae.fr>
;
; This program is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License
; as published by the Free Software Foundation; either version 2
; of the License, or (at your option) any later version.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software
; Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

   list p=16f877, st=OFF, x=OFF, n=0

   #include <p16f877.inc>

   __CONFIG _BODEN_OFF & _CP_OFF & _PWRTE_OFF & _WDT_OFF & _WRT_ENABLE_OFF & _HS_OSC & _DEBUG_OFF & _CPD_OFF & _LVP_OFF

; PWRTE and WRT cleared, meaning must be check

;*******************
; Static Definitions

#define POLY_HIGH    10h         ;MS Byte of X_Modem polynom 0x10
#define POLY_LOW     21h         ;LS Byte of X_Modem polynom 0x21

;Symbols and associated values
;for X_Modem Protocol flow control.

#define SOH          0x01        ;Start of Header
#define EOT          0x04        ;End of Transmission
#define ACK          0x06        ;Acknowledge
#define NAK          0x15        ;Not Acknowledge
#define ETB          0x17        ;End of Transmission Block (Return to Amulet OS mode)
#define CAN          0x18        ;Cancel (Force receiver to start sending C's)
#define CHR_C        0x43        ;ASCII 'C'

;SPBRG Register value for Baud Rate Generation

;BAUD_CONSTANT        EQU         0x09     ;115200 baud for 18.432 Mhz
                                           ;do not use debug only

BAUD_CONSTANT       EQU         0x13     ;57600 baud for 18.432 Mhz
;BAUD_CONSTANT       EQU         0x1D     ;38400 baud for 18.432 Mhz
;BAUD_CONSTANT       EQU         0x3B     ;19200 baud for 18.432 Mhz
;BAUD_CONSTANT       EQU         0x77     ;9600 baud for 18.432 Mhz

;Memory Register definitions

RAM_DMA_ADDRESS      EQU         0x7B
FLASH_HIGH_BYTE      EQU         0x7C
FLASH_LOW_BYTE       EQU         0x7D
W_REG_BKP            EQU         0x7E
STATUS_BKP           EQU         0x7F
CRC_HIGH             EQU         0x70     ;MS Byte of X_Modem CRC
CRC_LOW              EQU         0x71     ;LS Byte of X_Modem CRC
PACKET_#             EQU         0x72     ;X_Modem Packet #
EXT_RAM_HIGH         EQU         0x73     ;External RAM Bus address High Byte
EXT_RAM_LOW          EQU         0x74     ;External RAM Bus address Low Byte
CMD_STCK_STATE       EQU         0x6A     ;Serial Command Stack State
CMD_RCV_BYTE         EQU         0x6B     ;Received Command Byte
REG_RAM_1            EQU         0x6F     ;General 'Working' Register in Ram
REG_RAM_2            EQU         0x6E     ;General 'Working' Register in Ram
REG_RAM_3            EQU         0x6D     ;General 'Working' Register in Ram
REG_RAM_4            EQU         0x6C     ;General 'Working' Register in Ram


;*******************
; PRGM. PAGE MACROS
;*******************

Page0   MACRO ;macro to select PRGM. Memory Page0
                bcf PCLATH,4
                bcf PCLATH,3
                ENDM

Page1   MACRO ;macro to select PRGM. Memory Page1
                bcf PCLATH,4
                bsf PCLATH,3
                ENDM

Page2   MACRO ;macro to select PRGM. Memory Page2
                bsf PCLATH,4
                bcf PCLATH,3
                ENDM

Page3   MACRO ;macro to select PRGM. Memory Page3
                bsf PCLATH,4
                bsf PCLATH,3
                ENDM

;*******************
; BANK SEL. MACROS
;*******************

Bank0   MACRO ;macro to select bank 0
                bcf STATUS,RP0
                bcf STATUS,RP1
                ENDM

Bank1   MACRO ;macro to select bank 1
                bsf STATUS,RP0
                bcf STATUS,RP1
                ENDM

Bank2   MACRO ;macro to select bank 2
                bcf STATUS,RP0
                bsf STATUS,RP1
                ENDM

Bank3   MACRO ;macro to select bank 3
                bsf STATUS,RP0
                bsf STATUS,RP1
                ENDM

;*******************
; RESET VECTOR
;*******************

        org 0x00
        goto Start

;*******************
; INTERRUPT VECTOR
;*******************

; interruptions
        org 0x04
        retfie

;*******************
; PRGM. FUNCTIONS
;*******************

         org 0x800      ;PRGM. memory Page1

;__________________________________
;Global Port Setup
Global_Port_Setup:
                                        movwf W_REG_BKP              ;Save W register
                                        swapf STATUS,w               ;Swap STATUS / result in W
                                        movwf STATUS_BKP             ;Save "swapped" status

                                Bank1                    ;From Unknow to Bank1
                                ;clrf  TRISC            ;Port C as output port
                                clrf   TRISD             ;Port D as output port
                                movlw  0x07
                                movwf  ADCON1            ;PORTA & E as Digital I/O
                                clrf   TRISA             ;Port A as output port
                                clrf   TRISE             ;Port E as output port
                                movlw  0xFF
                                movwf  TRISB             ;Port B as input
                                                         ;Avoid Forcing RAM Data Bus if OE is enable
                                bcf    STATUS,RP0        ;change from bank1 to bank0
                                ;clrf  PORTC            ;00h sur Port C (CE not selected / RAM disabled)
                                clrf   PORTD             ;Ext. Ram Low_Byte
                                clrf   PORTA             ;Ext. Ram High_Byte
                                clrf   PORTE             ;Ext. Ram Ctrl

                                swapf STATUS_BKP,w           ; Swap old STATUS / result in W
                                movwf STATUS                 ; restore STATUS
                                        swapf W_REG_BKP,f            ; Swap L_Byte et H_Byte of the old W
                                                                              ; Z not affected
                                        swapf W_REG_BKP,w            ; Swap L_Byte and H_Byte / result in W
                                                                              ; W restored STATUS not affected
                                 return
;__________________________________

;Serial Port Setup
Serial_Setup:
                                        movwf W_REG_BKP                 ;Save W register
                                        swapf STATUS,w                  ;Swap STATUS / result in W
                                        movwf STATUS_BKP                ;Save "swapped" status

                                Bank1                    ;From Unknow to Bank1
                                ;movlw B'11000000'
                                ;iorwf TRISC,f          ;RC6 RC7 Port C as input pins
                                movlw  B'11111000'
                                movwf  TRISC             ;PORT C  Output Port
                                                         ;RAM Control Pins initialization
                                bcf    STATUS,RP0        ;Bank1 to Bank0
                                bsf    PORTC,1           ;/CE at High Level (Component (RAM) Not Selected)
                                bsf    PORTC,1           ;/OE at High Level (RAM Data Bus High Z)
                                bsf    PORTC,2           ;/WE at High Level (Not Write)

                                bsf    STATUS,RP0        ;Bank0 to Bank1
                                bcf    TXSTA,SYNC        ;asynchronous
                                movlw  BAUD_CONSTANT     ;set baud rate 115200 for 18.432 Mhz clock
                                movwf  SPBRG
                                bsf    TXSTA,BRGH        ;baud rate high speed option
                                bcf    STATUS,RP0        ;change from bank1 to bank0
                                bsf    RCSTA,SPEN        ;enable serial port pins
                                bsf    RCSTA,CREN        ;enable reception
                                bcf    RCSTA,SREN        ;unsed
                                bcf    RCSTA,TX9         ;8 bits Transmission
                                bsf    STATUS,RP0        ;change from bank0 to bank1
                                bcf    PIE1,TXIE         ;No interrupt TX
                                bcf    PIE1,RCIE         ;No interrupt RX
                                bcf    TXSTA,RX9         ;8 bits Reception
                                bsf    TXSTA,TXEN        ;enable transmission

                                        swapf STATUS_BKP,w           ; Swap old STATUS / result in W
                                        movwf STATUS                 ; restore STATUS
                                        swapf W_REG_BKP,f            ; Swap L_Byte et H_Byte of the old W
                                                                              ; Z not affected
                                        swapf W_REG_BKP,w               ; Swap L_Byte and H_Byte / result in W
                                                                              ; W restored STATUS not affected
                                 return
;__________________________________

Calculate_Frame_CRC:                   ;Calculate CRC of a 128 bytes Frame
                                                      ;located from 20h to 60h in Bank 0
                                                      ;and from A0 to E0 in Bank 1 and
                                                      ;store the result in CRC_HIGH (70h)
                                                      ;and CRC_LOW (71h).
                                                      ;X_Modem CRC polynom 1021h initial
                                                      ;value 00h.

                                        movwf W_REG_BKP                 ;Save W register
                                        swapf STATUS,w                  ;Swap STATUS / result in W
                                        movwf STATUS_BKP                ;Save "swapped" status

                                Bank0
                                bcf    STATUS,IRP        ;Indirect adressing in Bank0 & 1
                                movlw  0x20              ;initialize pointer
                                movwf  FSR               ;to RAM (Bank1)
                                addlw  0x40
                                movwf  REG_RAM_1         ;stop adress stored here
                                clrf   CRC_HIGH          ;reset CRC (MSB)
                                clrf   CRC_LOW           ;(LSB)
CRC_Bank_Zero:
                        movf   INDF,w           ;INDF register in W
                        xorwf  CRC_HIGH,f
                        movlw  0x08             ;# of bits to Roll
                        movwf  REG_RAM_2        ;in REG_RAM_2
                        call   Roll_CRC
                        incf   FSR,F            ;inc pointer
                        movf   FSR,w
                        xorwf  REG_RAM_1,w
                        btfss  STATUS,Z
                        goto   CRC_Bank_Zero
                        movlw  0xA0             ;initialize pointer
                        movwf  FSR              ;to RAM (Bank2)
                        addlw  0x40
                        movwf  REG_RAM_1        ;stop adress stored here
CRC_Bank_One:
                                    movf   INDF,w           ;INDF register in W
                                    xorwf  CRC_HIGH,f
                                    movlw  0x08             ;# of bits to Roll
                                    movwf  REG_RAM_2        ;in REG_RAM_2
                                    call   Roll_CRC
                                    incf   FSR,F ;inc pointer
                                    movf   FSR,w
                                    xorwf  REG_RAM_1,w
                                    btfss  STATUS,Z
                                    goto   CRC_Bank_One
                                    goto   End_CRC
Roll_CRC:
                        bcf    STATUS,C
                        rlf    CRC_LOW,f
                        rlf    CRC_HIGH,f
                        btfss  STATUS,C
                        goto   No_CRC_Xor
                        movlw  POLY_HIGH
                        xorwf  CRC_HIGH,f
                        movlw  POLY_LOW
                        xorwf  CRC_LOW,f
No_CRC_Xor:
                        decf   REG_RAM_2,f
                        btfss  STATUS, Z
                        goto   Roll_CRC
                        return
End_CRC:

                                        swapf STATUS_BKP,w           ; Swap old STATUS / result in W
                                        movwf STATUS                 ; restore STATUS
                                        swapf W_REG_BKP,f            ; Swap L_Byte et H_Byte of the old W
                                                                              ; Z not affected
                                        swapf W_REG_BKP,w            ; Swap L_Byte and H_Byte / result in W
                                                                              ; W restored STATUS not affected
;__________________________________

Read_Flash_Word:                       ;function read program memory
                                                      ;Read address specify in
                                                      ;0x7D (H_Byte) and 0x7E (L_Byte)
                                                      ;Put the result in these registers.

                                        movwf W_REG_BKP                 ;Save W register
                                        swapf STATUS,w                  ;Swap STATUS / result in W
                                        movwf STATUS_BKP                ;Save "swapped" status

                                Bank2                       ;From Unknow to Bank1
                                movf   FLASH_HIGH_BYTE,w
                                movwf  EEADRH
                                movf   FLASH_LOW_BYTE,w
                                movwf  EEADR
                                bsf    STATUS, RP0          ;Bank 3
                                bsf    EECON1, EEPGD        ;Point to PROGRAM memory (Flash)
                                bsf    EECON1, RD           ;EE Read
                                nop
                                nop
                                bcf    STATUS, RP0          ;Bank 2
                                movf   EEDATH,w             ;W = MS Byte of Program EEDATA
                                movwf  FLASH_HIGH_BYTE
                                movf   EEDATA,w             ;W = LS Byte of Program EEDATA
                                movwf  FLASH_LOW_BYTE
                                movf   STATUS_BKP,w

                                        swapf STATUS_BKP,w           ; Swap old STATUS / result in W
                                        movwf STATUS                 ; restore STATUS
                                        swapf W_REG_BKP,f            ; Swap L_Byte et H_Byte of the old W
                                                                              ; Z not affected
                                        swapf W_REG_BKP,w            ; Swap L_Byte and H_Byte / result in W
                                                                              ; W restored STATUS not affected
                                 return
;__________________________________

;*******************
         org 0x10

Start:
;        goto Read_All_Ext_Memory
                        Page1
                        call  Global_Port_Setup
                        call  Serial_Setup
                        bcf   PCLATH,3          ;PRGM. Memory Page 0

                        call  Intro             ;Display Start message

                        ;movlw   0x8C
                        ;call    CountDown

                        Bank0

         call  High_Z
         goto  Skip_RAM_Check

                        movlw '<'
                        call  Send_Byte


                  clrf    PORTA
                  clrf    PORTE
                  clrf    EXT_RAM_HIGH
                  movlw   0x00
                  movwf   EXT_RAM_LOW
                  call    Present_Address
;==================================
;Data Bus (PortB) Initialization (Write)
               Bank1                         ;From Unknow to Bank1
               clrf  TRISB                   ;Port B as output (Data Bus)
               bcf   STATUS,RP0              ;Bank1 to Bank0
;==================================

                                 movlw    'R'
                                 call     Write_Byte_Ext_RAM
                                 incf     PORTD,f
                                 movlw    'A'
                                 call     Write_Byte_Ext_RAM
                                 incf     PORTD,f
                                 movlw    'M'
                                 call     Write_Byte_Ext_RAM

;                                movlw    0x50
;                                call     CountDown
;==================================
;Data Bus (PortB) Initialization (Read)
                  clrf     PORTB             ;Port B Output Latches goes to '0'
                                             ;immediatly
                  Bank1
                  movlw    0xFF
                  movwf    TRISB             ;Port B as Input (Data Bus)
                  bcf      STATUS,RP0        ;Select Bank0
;==================================

                                 ;call    Read_Byte_Ext_RAM
                                 ;call    Send_Byte


                                 movlw    0x00
                                 movwf    EXT_RAM_LOW
                                 call     Present_Address
                                 call     Read_Byte_Ext_RAM
                                 call     Send_Byte


                                 movlw    0x01
                                 movwf    EXT_RAM_LOW
                                 call     Present_Address
                                 call     Read_Byte_Ext_RAM
                                 call     Send_Byte


                                 movlw    0x02
                                 movwf    EXT_RAM_LOW
                                 call     Present_Address
                                 call     Read_Byte_Ext_RAM
                                 call     Send_Byte


                                 movlw    '>'
                                 call     Send_Byte
                                 movlw    0x0D
                                 call     Send_Byte
                                 movlw    0x0A
                                 call     Send_Byte

                  Bank1
                  movlw    0xFF
                  movwf    TRISB                ;Port B as Input (Data Bus High Z)
                  bcf      STATUS,RP0           ;Select Bank0

Skip_RAM_Check

         goto Start_Com
;__________________________________________________________________
;__________________________________________________________________
;Routine for Commands
;command stack from 60h to 69h (10 Bytes) [RAM Bank0]
;command stack state at 6Ah [RAM Bank0]
;command receive byte at 6Bh [RAM Bank0]

Command:
         Bank0
         call     Get_Byte                   ;Read From serial Port
                                                   ;Received Character in W
         movwf    CMD_RCV_BYTE               ;Character stored in Ram
               ;Back Space (08h) is ignored
               ;in this version
               movlw    0x08
               xorwf    CMD_RCV_BYTE,w
               btfsc    STATUS,Z
               return
         movf     CMD_RCV_BYTE,w
         call     Send_Byte                  ;Perform echo of pressed Key
         movlw    0x0D                       ;End of Command, 'Carriage Return' (Return = CR)
         xorwf    CMD_RCV_BYTE,w             ;received data
         btfss    STATUS,Z
         goto     Space_key
         movlw    0x7F
         movwf    CMD_STCK_STATE
         return

Space_key:
         movlw    0x20                       ;ASCII code for SPACE (Hex)
         xorwf    CMD_RCV_BYTE,w             ;received data
         btfss    STATUS,Z
         goto     Tab_key
         movlw    0x00
         xorwf    CMD_STCK_STATE,w           ;No Character in Command Stack ?
         btfss    STATUS,Z
         goto     No_MAJ                     ;Not Empty, store SPACE key
                                                   ;Empty, SPACE ignore
         return

Tab_key:
         movlw    0x09                       ;ASCII code for TAB (HT) (Hex)
         xorwf    CMD_RCV_BYTE,w             ;received data
         btfss    STATUS,Z
         goto     MAJ
         movlw    0x00
         xorwf    CMD_STCK_STATE,w           ;No Character in Command Stack ?
         btfss    STATUS,Z
         goto     No_MAJ                     ;Not Empty, store SPACE key
                                                   ;Empty, SPACE ignore
         return

MAJ:
;Translate 'a' to 'z' in their MAJ equivalent 'A' to 'Z'
         movlw    0x85
         addwf    CMD_RCV_BYTE,w
         btfsc    STATUS,C                ; > 'z' ?
         goto     No_MAJ                  ; Yes, skip MAJ function
         movlw    0x9F
         addwf    CMD_RCV_BYTE,w
         btfss    STATUS,C                ; > 'a' ?
         goto     No_MAJ                  ; No, skip MAJ function
         movlw    0x20
         subwf    CMD_RCV_BYTE,f          ; Yes, translate to MAJ

No_MAJ:
         movf     CMD_STCK_STATE,w        ;CMD_STCK_STATE, replace by INDF
         addlw    0x60                    ;Point to Command Stack, add offset
         movwf    FSR
         movf     CMD_RCV_BYTE,w
         movwf    INDF
         incf     CMD_STCK_STATE,f
         movlw    0x0A
         xorwf    CMD_STCK_STATE,w
         btfss    STATUS,Z                ;Stack Full ?
         return
         movlw    0xFF
         movwf    CMD_STCK_STATE          ;Yes, CMD_STCK_STATE at FFh, Stack Full
         return
;__________________________________________________________________

Start_Com
         ;Clear RAM Bytes used by Commands stack.
         ;Ensure that IRP an STATUS is cleared
         ;to perform correct indirect adressing here.
                  Bank0
                  bcf   STATUS,IRP        ;Indirect adressing in Bank0 & 1
                  movlw 0x60              ;initialize pointer
                  movwf FSR               ;to RAM (Bank0)
                  addlw 0x0C
                  movwf REG_RAM_1         ;stop adress stored here
CLR_Command:
                  clrf  INDF              ;clear INDF register
                  incf  FSR,F             ;inc pointer
                  movf  FSR,w
                  xorwf REG_RAM_1,w
                  btfss STATUS,Z
                  goto  CLR_Command
                  call  Ready             ;Display "Ready>"
                                          ;Command Prompt
Get_Keys:
         call  Command
         btfss CMD_STCK_STATE,4     ;Not in field 60h to 69h
         goto  Get_Keys
         btfss CMD_STCK_STATE,7
         goto  Execute_CMD          ;Return Pressed

Stack_Full              call     Get_Byte                   ;Wait for a Character
                        movwf    CMD_RCV_BYTE               ;Keep key in memory
                              ;Back Space (08h) is ignored
                              ;in this version
                              movlw    0x08
                              xorwf    CMD_RCV_BYTE,w
                              btfsc    STATUS,Z
                              goto     Stack_Full
                        movf     CMD_RCV_BYTE,w
                        call     Send_Byte               ;Perform echo of pressed Key
                        movwf    CMD_RCV_BYTE            ;Received Character in W
                        movlw    0x0D                    ;End of Command, 'Carriage Return' (Return = CR)
                        xorwf    CMD_RCV_BYTE,w          ;received data
                        btfsc    STATUS,Z
                        goto     Execute_CMD
                        movlw    0x20                    ;'Space' Key
                        xorwf    CMD_RCV_BYTE,w          ;received data
                        btfsc    STATUS,Z
                        goto     Stack_Full              ;no effect
                        movlw    0x09                    ;'Tab' Key
                        xorwf    CMD_RCV_BYTE,w          ;received data
                        btfsc    STATUS,Z
                        goto     Stack_Full              ;no effect
                        movlw    0x01                    ;Any other key
                        movwf    0x69
                        goto     Stack_Full
;__________________________________________________________________
;__________________________________________________________________

Execute_CMD:
         clrw
         ;'Enter' Key pressed with No Command
         ;Command Stack Byte 1 (Should not be 00h)
         xorwf    0x60,w
         btfsc    STATUS,Z
         goto     Empty_Cmd
         movlw    0x01                    ;Character Key pressed after
         xorwf    0x69,w                  ;Command Stack overflow ?
         btfss    STATUS,Z
         ;goto    Display_Stack
         goto     Search_Cmd
         call     Wrong_Cmd               ;Display error message
         goto     Start_Com               ;Ready for a new command
Empty_Cmd               movlw    0x0A                    ;'Line Feed'
                        call     Send_Byte
                        ;call    Send_Byte               ;Twice
                        movlw    0x0D                    ;'CR'
                        call     Send_Byte
                        goto     Start_Com               ;Ready for a new command
;_ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _

Search_Cmd
                        call     Cmd_Mask
                        call     Cmd_Group
                        goto     Start_Com               ;Ready for a new command
;_  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _
;_  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _

;Groups are a set of Commands beginning by the same letter.
;Group is determined by the first letter of the stack (60h).
Cmd_Group
                                 movlw    'C'               ;Begin Letter
                                 xorwf    0x60,w            ;Command Stack Byte 1
                                 btfsc    STATUS,Z
                                 goto     Group_C
                                 movlw    'D'               ;Begin Letter
                                 xorwf    0x60,w            ;Command Stack Byte 1
                                 btfsc    STATUS,Z
                                 goto     Group_D
                                 movlw    'F'               ;Begin Letter
                                 xorwf    0x60,w            ;Command Stack Byte 1
                                 btfsc    STATUS,Z
                                 goto     Group_F
                                 movlw    'H'               ;Begin Letter
                                 xorwf    0x60,w            ;Command Stack Byte 1
                                 btfsc    STATUS,Z
                                 goto     Group_H
                                 movlw    'R'               ;Begin Letter
                                 xorwf    0x60,w            ;Command Stack Byte 1
                                 btfsc    STATUS,Z
                                 goto     Group_R
                                 call     Wrong_Cmd         ;Display error message
                                 return
;_  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _

Group_C
         ;looks for help command ("CLR") here / Address 0x1FE8
                        movlw    0x1F
                        movwf    FLASH_HIGH_BYTE
                        movlw    0xE8                       ;Starts at 0x1FE8
                        movwf    FLASH_LOW_BYTE
                        call     Cmd_Match
                  ;Matched 'R'ight or 'W'rong in W
                        xorlw    'R'
                        btfsc    STATUS,Z
                        goto     CLR_DSPLY
                  ;looks for next here
                           call     Wrong_Cmd
                           return

Group_D
         ;looks for help command ("DMP") here / Address 0x1FE0
                        movlw    0x1F
                        movwf    FLASH_HIGH_BYTE
                        movlw    0xE0                       ;Starts at 0x1FE0
                        movwf    FLASH_LOW_BYTE
                        call     Cmd_Match
                  ;Matched 'R'ight or 'W'rong in W
                        xorlw    'R'
                        btfsc    STATUS,Z
                        goto     DMP_DSPLY
                  ;looks for next here
                           call     Wrong_Cmd
                           return

Group_F
         ;looks for help command ("FIL") here / Address 0x1FD8
                        movlw    0x1F
                        movwf    FLASH_HIGH_BYTE
                        movlw    0xD8                       ;Starts at 0x1FD8
                        movwf    FLASH_LOW_BYTE
                        call     Cmd_Match
                  ;Matched 'R'ight or 'W'rong in W
                        xorlw    'R'
                        btfsc    STATUS,Z
                        goto     FIL_DSPLY
                  ;looks for next here
                           call     Wrong_Cmd
                           return

Group_H
         ;looks for help command ("HLP") here / Address 0x1FF8
                        movlw    0x1F
                        movwf    FLASH_HIGH_BYTE
                        movlw    0xF8                       ;Starts at 0x1FF8
                        movwf    FLASH_LOW_BYTE
                        call     Cmd_Match
                  ;Matched 'R'ight or 'W'rong in W
                        xorlw    'R'
                        btfsc    STATUS,Z
                        goto     HLP_DSPLY
                  ;looks for help command ("HGZ") here / Address 0x1FD0
                        movlw    0x1F
                        movwf    FLASH_HIGH_BYTE
                        movlw    0xD0                       ;Starts at 0x1FD0
                        movwf    FLASH_LOW_BYTE
                        call     Cmd_Match
                  ;Matched 'R'ight or 'W'rong in W
                        xorlw    'R'
                        btfsc    STATUS,Z
                        goto     HGZ_DSPLY
                  ;looks for next here
                           call     Wrong_Cmd
                           return

Group_R
         ;looks for reset command ("RST") here / Address 0x1FF0
                        movlw    0x1F
                        movwf    FLASH_HIGH_BYTE
                        movlw    0xF0                       ;Starts at 0x1FF0
                        movwf    FLASH_LOW_BYTE
                        call     Cmd_Match
                  ;Matched 'R'ight or 'W'rong in W
                        xorlw    'R'
                        btfsc    STATUS,Z
                        goto     0x00                       ;Go to RESET Vector
                                                            ;No problem with Stack
                                                            ;This is a Circular Stack.
                  ;looks for next here
                           call     Wrong_Cmd
                           return


;+++++++++++++++++++++++++++
HLP_DSPLY
         call     Help
         goto     Start_Com                     ;Ready for a new command
;+++++++++++++++++++++++++++
CLR_DSPLY
         movlw    0x0c                          ;Clear Screen
         call     Send_Byte
         goto     Start_Com                     ;Ready for a new command
;+++++++++++++++++++++++++++
DMP_DSPLY
         goto     Send_Srl
         ;call    Not_Implmnt
         goto     Start_Com                     ;Ready for a new command
;+++++++++++++++++++++++++++
FIL_DSPLY
         call     Get_Srl
         ;call    Not_Implmnt
         goto     Start_Com                     ;Ready for a new command
;+++++++++++++++++++++++++++
HGZ_DSPLY
         call     High_Z
         call     OK_Dsply
         ;call    Not_Implmnt
         goto     Start_Com                     ;Ready for a new command
;+++++++++++++++++++++++++++
;_  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _
;_  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _

Cmd_Mask:
         ;Change 'Tab' & 'Space' characters
         ;in Command Stack by 00h.
                  bcf   STATUS,IRP        ;Indirect adressing in Bank0 & 1
                  movlw 0x60              ;initialize pointer
                  movwf FSR               ;to Command Stack in RAM (Bank1)
                  addlw 0x0A
                  movwf REG_RAM_1         ;stop adress stored here
Mask_Stack:
                  movf  INDF,w            ;Pointed char in W reg
                  movwf REG_RAM_2
                  movlw 0x20              ;Character is Space ?
                  xorwf REG_RAM_2,w
                  btfss STATUS,Z
                  goto  Tab_Check         ;No, check 'Tab' case
                  movlw 0x00
                  movwf INDF              ;Yes, change Space for 01h
                  goto  Mask_Next
Tab_Check:
                  movlw 0x09              ;Character is 'Space' ?
                  xorwf REG_RAM_2,w
                  btfss STATUS,Z
                  goto  Mask_Next         ;No, go to check next character
                  movlw 0x00
                  movwf INDF              ;Yes, change 'Tab' for 01h
Mask_Next:
                  incf  FSR,F ;inc pointer
                  movf  FSR,w
                  xorwf REG_RAM_1,w
                  btfss STATUS,Z
                  goto  Mask_Stack
                  return
;_ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _
;_ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _

;Make a comparison beetween Command Stack and Command in Flash
;stored at address specified by FLASH_HIGH_BYTE and FLASH_LOW_BYTE.
;Place 'R' if the two stacks match, 'W' otherwise.
Cmd_Match            movf  FLASH_LOW_BYTE,w
                     movwf REG_RAM_1
                     movf  FLASH_LOW_BYTE,w
                     movwf REG_RAM_2
                     movlw 0x05
                     addwf REG_RAM_2,f          ;Stop condition at stop address + 1
                     ;bcf  STATUS,IRP           ;Indirect adressing in Bank0 & 1
                     movlw 0x60                 ;initialize pointer
                     movwf FSR                  ;to Command Stack in RAM (Bank1)
Cmd_Match_Read_Next_Byte_Flash:
                              call  Flash_Match
                              movwf REG_RAM_3
                              movlw 'W'
                              xorwf REG_RAM_3,f
                              btfsc STATUS,Z
                              return
                              incf  REG_RAM_1,f
                              movlw 0x1F                 ;Restore Flash H_Address
                              movwf FLASH_HIGH_BYTE
                              movf  REG_RAM_1,w
                              movwf FLASH_LOW_BYTE
                              xorwf REG_RAM_2,w
                              btfss STATUS,Z
                              goto  Cmd_Match_Read_Next_Byte_Flash
                              movlw 'R'
                              return
Flash_Match             Page1
                        call  Read_Flash_Word
                        bcf   PCLATH,3                   ;PRGM. Memory Page 0
                        bcf   STATUS,C
                        rlf   FLASH_LOW_BYTE,w
                        rlf   FLASH_HIGH_BYTE,w
                  ;Valid High Byte is in W
                        xorwf INDF,w
                        btfss STATUS,Z
                        goto  Wrong_Byte
                        incf  FSR,f
                        bcf   FLASH_LOW_BYTE,7
                        movf  FLASH_LOW_BYTE,w
                  ;Valid Low Byte is in W
                        xorwf INDF,w
                        btfss STATUS,Z
                        goto  Wrong_Byte
                        incf  FSR,f
                        return
Wrong_Byte              movlw 'W'
                        return
;_ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _
;_ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _

Display_Stack
;To debug the program (Command Stack)
                  Bank0
                  movlw 0x0D                    ;CR
                  call  Send_Byte
                  movlw 0x0A                    ;LF
                  call  Send_Byte
         movf  0x60,w
         call  Send_Byte
         movf  0x61,w
         call  Send_Byte
         movf  0x62,w
         call  Send_Byte
         movf  0x63,w
         call  Send_Byte
         movf  0x64,w
         call  Send_Byte
         movf  0x65,w
         call  Send_Byte
         movf  0x66,w
         call  Send_Byte
         movf  0x67,w
         call  Send_Byte
         movf  0x68,w
         call  Send_Byte
         movf  0x69,w
         call  Send_Byte
         goto  End_CMD

End_CMD:
;__________________________________________________________________
;__________________________________________________________________
;Configures all pins of Port A, B, E, D connected to External RAM
;as input (High Z).
High_Z
                        Bank1                         ;From Unknow to Bank1
                        movlw 0xFF
                        movwf TRISA                   ;Port A as input (Address Bus)
                        movwf TRISB                   ;Port B as input (Data Bus)
                        movwf TRISD                   ;Port D as input (Address Bus)
                        movwf TRISE                   ;Port E as input (Address Bus)
                        bsf   TRISC,0                 ;Port C (RAM controls)
                        bsf   TRISC,1                 ;Port C (RAM controls)
                        bsf   TRISC,2                 ;Port C (RAM controls)
                        bcf   STATUS,RP0              ;Bank1 to Bank0
                        return                        ;To Commands Lines
;__________________________________________________________________

;_  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _
;_  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _
Get_Srl_beg

;Active Wait for 'C'
Get_Srl
;==================================
;Data Bus (PortB) Initialization (Write)
                  Bank1                         ;From Unknow to Bank1
                  clrf  TRISB                   ;Port B as output (Data Bus)
                  clrf  TRISA                   ;Port A as output (Address Bus)
                  clrf  TRISD                   ;Port D as output (Address Bus)
                  clrf  TRISE                   ;Port E as output (Address Bus)
                  bcf   TRISC,0                 ;Port C (RAM controls)
                  bcf   TRISC,1                 ;Port C (RAM controls)
                  bcf   TRISC,2                 ;Port C (RAM controls)
                  bcf   STATUS,RP0              ;Bank1 to Bank0
                           ;RAM Control Pins initialization
                           bsf   PORTC,1                 ;/CE at High Level (Component (RAM) Not Selected)
                           bsf   PORTC,1                 ;/OE at High Level (RAM Data Bus High Z)
                           bsf   PORTC,2                 ;/WE at High Level (Not Write)

;==================================
;                 Bank0
                  clrf  EXT_RAM_HIGH
                  clrf  EXT_RAM_LOW
                  clrf  REG_RAM_1
                  clrf  REG_RAM_3
                  clrf  TMR1L
                  clrf  TMR1H
                  movlw B'00000000'
                  movwf T1CON
                           movlw 0x0D                       ;'CR'
                           call  Send_Byte
                           movlw 0x0A                       ;'Line Feed'
                           call  Send_Byte
Active_C
                  movlw 'C'
                  call  Send_Byte
                  bsf   T1CON,TMR1ON                        ;Start Timer
C_TMR1_Overflow            btfsc    PIR1,5                  ;A Byte has come ?
                           goto     Dat_Detected
                           bcf      STATUS,5
                           bcf      STATUS,6
                           btfss    PIR1,TMR1IF
                           goto     C_TMR1_Overflow
                           bcf      PIR1,TMR1IF
                           incf     REG_RAM_1,f
                           movlw    0xEA                    ;~5 Seconds
                           xorwf    REG_RAM_1,w
                           btfss    STATUS,Z
                           goto     C_TMR1_Overflow
                           goto     Active_C
Dat_Detected
               movf     RCREG,w                             ;Must be 0x01 <SOH>
               xorlw    0x01
               btfss    STATUS,Z
               goto     Invalid_SOH
;_  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _

Get_Next_Frm:
                           ;call    Get_Byte    ;Skip Header
                           call     Get_Byte    ;
                           call     Get_Byte    ;
;Fill from 20h to 60h in Bank0
;Fill from A0h to E0h in Bank1

         ;Ensure that IRP in STATUS is cleared
         ;to perform correct indirect adressing here.
               Bank0
               bcf   STATUS,IRP        ;Indirect adressing in Bank0 & 1
               movlw 0x20              ;initialize pointer
               movwf FSR               ;to RAM (Bank0)
               addlw 0x40
               movwf REG_RAM_1         ;stop adress stored here
Store_Bank_Zero:
               call  Get_Byte
               movwf INDF              ;Received Byte stored in INDF
               incf  FSR,F             ;inc pointer
               movf  FSR,w
               xorwf REG_RAM_1,w
               btfss STATUS,Z
               goto  Store_Bank_Zero
               movlw 0xA0              ;initialize pointer
               movwf FSR               ;to RAM (Bank1)
               addlw 0x40
               movwf REG_RAM_1         ;stop adress stored here
Store_Bank_One:
               call  Get_Byte
               movwf INDF              ;Received Byte stored in INDF
               incf  FSR,F             ;inc pointer
               movf  FSR,w
               xorwf REG_RAM_1,w
               btfss STATUS,Z
               goto  Store_Bank_One
                           call     Get_Byte       ;Skip Checksum
                           call     Get_Byte       ;
;                          movlw    0x06           ;Send Ack
;                          call     Send_Byte
;_  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _

;                          call     Get_Byte       ;Skip EOT
;                          movlw    0x06           ;Send Ack
;                          call     Send_Byte

;_  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _
;_  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _

;_  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _
;_  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _

Write_Banks_To_EXT_RAM:                   ;Write 128 Bytes from Internal RAM
                                          ;to External RAM
                  Bank0
                  bcf   STATUS,IRP                 ;Indirect adressing in Bank0 & 1
                  movlw 0x20                       ;initialize pointer
                  movwf FSR                        ;to RAM (Bank0)
                  addlw 0x40
                  movwf REG_RAM_1                  ;stop adress stored here
Write_Bank_Zero_To_Ext_Ram:
                  call  Present_Address
;                 movwf PORTB                      ;Data to PORTB, parallel Data Bus
                  movf  INDF,w
                  call  Write_Byte_Ext_RAM
                  incf  EXT_RAM_LOW,f
                  btfsc STATUS, Z                  ;EXT_RAM_LOW has overflowed ?
                  incf  EXT_RAM_HIGH,f             ;Yes, increment EXT_RAM_HIGH
                  incf  FSR,F                      ;inc pointer
                  movf  FSR,w
                  xorwf REG_RAM_1,w
                  btfss STATUS,Z
                  goto  Write_Bank_Zero_To_Ext_Ram
                  movlw 0xA0                       ;initialize pointer
                  movwf FSR                        ;to RAM (Bank1)
                  addlw 0x40
                  movwf REG_RAM_1                  ;stop adress stored here
Write_Bank_One_To_Ext_Ram:
                  call  Present_Address
;                 movwf PORTB                      ;Data to PORTB, parallel Data Bus
                  movf  INDF,w
                  call  Write_Byte_Ext_RAM
                  incf  EXT_RAM_LOW,f
                  btfsc STATUS, Z                  ;EXT_RAM_LOW has overflowed ?
                  incf  EXT_RAM_HIGH,f             ;Yes, increment EXT_RAM_HIGH
                  incf  FSR,F                      ;inc pointer
                  movf  FSR,w
                  xorwf REG_RAM_1,w
                  btfss STATUS,Z
                  goto  Write_Bank_One_To_Ext_Ram
;_  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _
;_  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _
;_  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _

                        incf  REG_RAM_3,f
                        btfsc STATUS,Z
                        goto  End_Acq
                        movlw ACK                        ;Send Ack (0x06)
                        call  Send_Byte
                        call  Get_Byte                   ;Skip Header
                        goto  Get_Next_Frm

End_Acq
                              movlw 0x06                       ;Send Ack
                              call  Send_Byte
                              call  Get_Byte                   ;Skip EOT
                              movlw 0x06                       ;Send Ack
                              call  Send_Byte
                              bcf   T1CON,TMR1ON               ;Stop Timer

                                 call  RAM_Filled

                                 return                        ;To Commands Lines

Invalid_SOH
                                 movlw 0x0D                    ;'CR'
                                 call  Send_Byte
                                 movlw 0x0A                    ;'Line Feed'
                                 call  Send_Byte
                                 call  Optn_Cancel

                                 return                        ;To Commands Lines
;_  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _
;_  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _
;_  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _


;_  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _
;_  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _
;_  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _

Send_Srl


;==================================
;Data Bus (PortB) Initialization (Read)
                  Bank1
                  movlw 0xFF
                  movwf TRISB                   ;Port B as Input (Data Bus)
                  clrf  TRISA                   ;Port A as output (Address Bus)
                  clrf  TRISD                   ;Port D as output (Address Bus)
                  clrf  TRISE                   ;Port E as output (Address Bus)
                  bcf   TRISC,0                 ;Port C (RAM controls)
                  bcf   TRISC,1                 ;Port C (RAM controls)
                  bcf   TRISC,2                 ;Port C (RAM controls)
                  bcf   STATUS,RP0              ;Select Bank0
            ;RAM Control Pins initialization
                              bsf   PORTC,1        ;/CE at High Level (Component (RAM) Not Selected)
                              bsf   PORTC,1        ;/OE at High Level (RAM Data Bus High Z)
                              bsf   PORTC,2        ;/WE at High Level (Not Write)

;==================================

                        clrf  EXT_RAM_HIGH
                        clrf  EXT_RAM_LOW
                        clrf  REG_RAM_1
                        clrf  REG_RAM_3
;                       clrf  TMR1L
;                       clrf  TMR1H
;                       movlw B'00000000'
;                       movwf T1CON
                                          movlw 0x0D           ;'CR'
                                          call  Send_Byte
                                          movlw 0x0A           ;'Line Feed'
                                          call  Send_Byte
                        call  Get_Byte
                        xorlw 'C'
                        btfsc STATUS,Z
                        goto  Read_All_Ext_Memory
                        call  Optn_Cancel

                  return                                       ;To Commands Lines

;_  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _

Read_All_Ext_Memory
                        movlw SOH                              ;Start of Header
                        call  Send_Byte
                        incf  REG_RAM_3,w                      ;Packet #
                        call  Send_Byte
                        movwf REG_RAM_4
                        comf  REG_RAM_4,w
                        call  Send_Byte

Store_External_RAM:                       ;Store 128 Bytes from External RAM
                                          ;to Internal RAM
                  bcf   STATUS,IRP                    ;Indirect adressing in Bank0 & 1
                  movlw 0x20                          ;initialize pointer
                  movwf FSR                           ;to RAM (Bank1)
                  addlw 0x40
                  movwf REG_RAM_1                     ;stop adress stored here
Read_Bank_Zero_From_Ext_Ram:
                  call  Present_Address
                  call  Read_Byte_Ext_RAM
                  movwf INDF
                  incf  EXT_RAM_LOW,f
                  btfsc STATUS, Z                     ;EXT_RAM_LOW has overflowed ?
                  incf  EXT_RAM_HIGH,f                ;Yes, increment EXT_RAM_HIGH
                  incf  FSR,F                         ;inc pointer
                  movf  FSR,w
                  xorwf REG_RAM_1,w
                  btfss STATUS,Z
                  goto  Read_Bank_Zero_From_Ext_Ram
                  movlw 0xA0                          ;initialize pointer
                  movwf FSR                           ;to RAM (Bank2)
                  addlw 0x40
                  movwf REG_RAM_1                     ;stop adress stored here
Read_Bank_One_From_Ext_Ram:
                  call  Present_Address
                  call  Read_Byte_Ext_RAM
                  movwf INDF
                  incf  EXT_RAM_LOW,f
                  btfsc STATUS, Z                     ;EXT_RAM_LOW has overflowed ?
                  incf  EXT_RAM_HIGH,f                ;Yes, increment EXT_RAM_HIGH
                  incf  FSR,F                         ;inc pointer
                  movf  FSR,w
                  xorwf REG_RAM_1,w
                  btfss STATUS,Z
                  goto  Read_Bank_One_From_Ext_Ram

;_  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _
;_  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _  _

;Read from 20h to 60h in Bank0
;Read from A0h to E0h in Bank1

         ;Ensure that IRP in STATUS is cleared
         ;to perform correct indirect adressing here.
                  Bank0
                  bcf   STATUS,IRP                 ;Indirect adressing in Bank0 & 1
                  movlw 0x20                       ;initialize pointer
                  movwf FSR                        ;to RAM (Bank1)
                  addlw 0x40
                  movwf REG_RAM_1                  ;stop adress stored here
Read_Bank_Zero:
                  movf  INDF,w                     ;Read Byte from INDF
                  call  Send_Byte
                  incf  FSR,F ;inc pointer
                  movf  FSR,w
                  xorwf REG_RAM_1,w
                  btfss STATUS,Z
                  goto  Read_Bank_Zero
                  movlw 0xA0                       ;initialize pointer
                  movwf FSR                        ;to RAM (Bank2)
                  addlw 0x40
                  movwf REG_RAM_1                  ;stop adress stored here
Read_Bank_One:
                  movf  INDF,w                     ;Read Byte from INDF
                  call  Send_Byte
                  incf  FSR,F                      ;inc pointer
                  movf  FSR,w
                  xorwf REG_RAM_1,w
                  btfss STATUS,Z
                  goto  Read_Bank_One

                        Page1
                        call  Calculate_Frame_CRC
                        bcf   PCLATH,3                            ;PRGM. Memory Page 0

                        movf  CRC_HIGH,w
                        call  Send_Byte
                        movf  CRC_LOW,w
                        call  Send_Byte
                        call  Get_Byte                            ;Ignore 'Ack'
                                       incf  REG_RAM_3,f
                                       btfss STATUS,Z
                                       goto  Read_All_Ext_Memory
                        movlw EOT                                 ;End of Transmission (04h)
                        call  Send_Byte
                        call  Get_Byte                            ;Ignore 'Ack'
                        call  RAM_Dumped

                  return                                          ;To Commands Lines

                        goto  Skip_Functions

;__________________________________
;Start message.
;Clear Screen and display message
;located at 0x1800 (PRGM. Memory
;Page 3)
Intro                   movlw 0x0c                                   ;Clear Screen
                        call  Send_Byte
                        movlw 0x00                                   ;Starts at 0x1800
                        movwf REG_RAM_1
                        movlw 0x17                                   ;Stops at 0x1816
                        movwf REG_RAM_2                              ;Stop condition at stop address + 1
                        movlw 0x18
                        movwf FLASH_HIGH_BYTE
                        clrf  FLASH_LOW_BYTE
START_Read_Next_Byte_Flash:
                                 call  ASCII_7_to_8
                                 incf  REG_RAM_1,f
                                 movlw 0x18
                                 movwf FLASH_HIGH_BYTE
                                 movf  REG_RAM_1,w
                                 movwf FLASH_LOW_BYTE
                                 xorwf REG_RAM_2,w
                                 btfss STATUS,Z
                                 goto  START_Read_Next_Byte_Flash
                                 return

;__________________________________
;Displays 'Ready>' / begin a
;command line.
Ready                   movlw 0x20                                   ;Starts at 0x1820
                        movwf FLASH_LOW_BYTE
                        movwf REG_RAM_1
                        movlw 0x23                                   ;Stops at 0x1822
                        movwf REG_RAM_2                              ;Stop condition at stop address + 1
                        movlw 0x18
                        movwf FLASH_HIGH_BYTE
READY_Read_Next_Byte_Flash:
                                 call  ASCII_7_to_8
                                 incf  REG_RAM_1,f
                                 movlw 0x18
                                 movwf FLASH_HIGH_BYTE
                                 movf  REG_RAM_1,w
                                 movwf FLASH_LOW_BYTE
                                 xorwf REG_RAM_2,w
                                 btfss STATUS,Z
                                 goto  READY_Read_Next_Byte_Flash
                                 return
;__________________________________
;Displays 'Cartridge Filled'.
RAM_Filled              movlw 0xB0                                   ;Starts at 0x18B0
                        movwf FLASH_LOW_BYTE
                        movwf REG_RAM_1
                        movlw 0xBB                                   ;Stops at 0x18BA
                        movwf REG_RAM_2                              ;Stop condition at stop address + 1
                        movlw 0x18
                        movwf FLASH_HIGH_BYTE
RAM_Filled_Read_Next_Byte_Flash:
                                 call  ASCII_7_to_8
                                 incf  REG_RAM_1,f
                                 movlw 0x18
                                 movwf FLASH_HIGH_BYTE
                                 movf  REG_RAM_1,w
                                 movwf FLASH_LOW_BYTE
                                 xorwf REG_RAM_2,w
                                 btfss STATUS,Z
                                 goto  RAM_Filled_Read_Next_Byte_Flash
                                 return
;__________________________________
;Displays 'Cartridge Dumped'.
RAM_Dumped              movlw 0xC0                                   ;Starts at 0x18C0
                        movwf FLASH_LOW_BYTE
                        movwf REG_RAM_1
                        movlw 0xCA                                   ;Stops at 0x18C9
                        movwf REG_RAM_2                              ;Stop condition at stop address + 1
                        movlw 0x18
                        movwf FLASH_HIGH_BYTE
RAM_Dumped_Read_Next_Byte_Flash:
                                 call  ASCII_7_to_8
                                 incf  REG_RAM_1,f
                                 movlw 0x18
                                 movwf FLASH_HIGH_BYTE
                                 movf  REG_RAM_1,w
                                 movwf FLASH_LOW_BYTE
                                 xorwf REG_RAM_2,w
                                 btfss STATUS,Z
                                 goto  RAM_Dumped_Read_Next_Byte_Flash
                                 return
;__________________________________
;Displays 'Ok'.
OK_Dsply                movlw 0xA0                                   ;Starts at 0x18A0
                        movwf FLASH_LOW_BYTE
                        movwf REG_RAM_1
                        movlw 0xA4                                   ;Stops at 0x18A3
                        movwf REG_RAM_2                              ;Stop condition at stop address + 1
                        movlw 0x18
                        movwf FLASH_HIGH_BYTE
OK_Dsply_Read_Next_Byte_Flash:
                                 call  ASCII_7_to_8
                                 incf  REG_RAM_1,f
                                 movlw 0x18
                                 movwf FLASH_HIGH_BYTE
                                 movf  REG_RAM_1,w
                                 movwf FLASH_LOW_BYTE
                                 xorwf REG_RAM_2,w
                                 btfss STATUS,Z
                                 goto  OK_Dsply_Read_Next_Byte_Flash
                                 return
;__________________________________
;Displays 'Operation Cancelled'.
Optn_Cancel             movlw 0xD0                                   ;Starts at 0x18D0
                        movwf FLASH_LOW_BYTE
                        movwf REG_RAM_1
                        movlw 0xDC                                   ;Stops at 0x18DC
                        movwf REG_RAM_2                              ;Stop condition at stop address + 1
                        movlw 0x18
                        movwf FLASH_HIGH_BYTE
Optn_Cancel_Read_Next_Byte_Flash:
                                 call  ASCII_7_to_8
                                 incf  REG_RAM_1,f
                                 movlw 0x18
                                 movwf FLASH_HIGH_BYTE
                                 movf  REG_RAM_1,w
                                 movwf FLASH_LOW_BYTE
                                 xorwf REG_RAM_2,w
                                 btfss STATUS,Z
                                 goto  Optn_Cancel_Read_Next_Byte_Flash
                                 return
;__________________________________
;Displays 'Wrong Command'.
Wrong_Cmd               movlw 0x26                                   ;Starts at 0x1826
                        movwf FLASH_LOW_BYTE
                        movwf REG_RAM_1
                        movlw 0x30                                   ;Stops at 0x182F
                        movwf REG_RAM_2                              ;Stop condition at stop address + 1
                        movlw 0x18
                        movwf FLASH_HIGH_BYTE
W_CMD_Read_Next_Byte_Flash:
                                 call  ASCII_7_to_8
                                 incf  REG_RAM_1,f
                                 movlw 0x18
                                 movwf FLASH_HIGH_BYTE
                                 movf  REG_RAM_1,w
                                 movwf FLASH_LOW_BYTE
                                 xorwf REG_RAM_2,w
                                 btfss STATUS,Z
                                 goto  W_CMD_Read_Next_Byte_Flash
                                 return
;__________________________________
;Displays 'Not Implemented'.
Not_Implmnt             movlw 0x30                                   ;Starts at 0x1830
                        movwf FLASH_LOW_BYTE
                        movwf REG_RAM_1
                        movlw 0x3B                                   ;Stops at 0x183A
                        movwf REG_RAM_2                              ;Stop condition at stop address + 1
                        movlw 0x18
                        movwf FLASH_HIGH_BYTE
Not_Implmnt_Read_Next_Byte_Flash:
                                 call  ASCII_7_to_8
                                 incf  REG_RAM_1,f
                                 movlw 0x18
                                 movwf FLASH_HIGH_BYTE
                                 movf  REG_RAM_1,w
                                 movwf FLASH_LOW_BYTE
                                 xorwf REG_RAM_2,w
                                 btfss STATUS,Z
                                 goto  Not_Implmnt_Read_Next_Byte_Flash
                                 return
;__________________________________
;Displays Help Command.
Help                    movlw 0x40                                   ;Starts at 0x1840
                        movwf FLASH_LOW_BYTE
                        movwf REG_RAM_1
                        movlw 0x96                                   ;Stops at 0x1895
                        movwf REG_RAM_2                              ;Stop condition at stop address + 1
                        movlw 0x18
                        movwf FLASH_HIGH_BYTE
Help_Read_Next_Byte_Flash:
                                 call  ASCII_7_to_8
                                 incf  REG_RAM_1,f
                                 movlw 0x18
                                 movwf FLASH_HIGH_BYTE
                                 movf  REG_RAM_1,w
                                 movwf FLASH_LOW_BYTE
                                 xorwf REG_RAM_2,w
                                 btfss STATUS,Z
                                 goto  Help_Read_Next_Byte_Flash
                                 return
;__________________________________
;Convert two 7 bits Characters codes
;into two 8 bits Characters codes
;and send them trought serial port.
;7 Bits codes are in Program Memory.
ASCII_7_to_8                     Page1
                                 call  Read_Flash_Word
                                 bcf   PCLATH,3                      ;PRGM. Memory Page 0
                                 bcf   STATUS,C
                                 rlf   FLASH_LOW_BYTE,w
                                 rlf   FLASH_HIGH_BYTE,w
                                 call  Send_Byte                     ;Send High Byte
                                 bcf   FLASH_LOW_BYTE,7
                                 movf  FLASH_LOW_BYTE,w
                                 call  Send_Byte                     ;Send Low Byte
                                 return
;__________________________________
Present_Address:              ;Fit registers EXT_RAM_LOW & EXT_RAM_HIGH
                              ;to PORT_D (LOW) & PORT_A and PORT_E (HIGH)
         movf  EXT_RAM_LOW,w
         movwf PORTD
         movf  EXT_RAM_HIGH,w
;Fit 'EXT_RAM_HIGH' in PortA (A0-A3) and PortE (E0-E2)
;Ensure Bank0 is selected
         andlw 0x0F
         movwf PORTA
         swapf EXT_RAM_HIGH,w
;        andlw 0x07
         movwf PORTE
         return
;__________________________________
;Time for Countdown should be
;specified in W before calling
;this function.
;1 Second Countdown 46h (18.432Mhz)
;2 Second Countdown 8Ch (18.432Mhz)
CountDown               Bank0
                        movwf REG_RAM_2
                        clrf  REG_RAM_1
                        clrf  TMR1L
                        clrf  TMR1H
                        movlw B'00000000'
                        movwf T1CON
                        bsf   T1CON,TMR1ON

TMR1_Overflow                    bcf   STATUS,5
                                 bcf   STATUS,6
                                 btfss PIR1,TMR1IF
                                 goto  TMR1_Overflow
                                 bcf   PIR1,TMR1IF
                                 incf  REG_RAM_1,f
                                 movf  REG_RAM_2,w
                                 xorwf REG_RAM_1,w
                                 btfss STATUS,Z
                                 goto  TMR1_Overflow
                        return
;__________________________________
;Read a Byte from Ext.RAM
;Read Byte stored in W
Read_Byte_Ext_RAM:
                  bsf   PORTC,1                 ;/OE High (Output Not Enable)
                  bcf   PORTC,0                 ;/CE Low / Select component (RAM)
                  bcf   PORTC,1                 ;/OE Low (Read)
                  nop
                  nop
                  movf  PORTB,w                 ;Data from PORTB to W, parallel Data Bus
                  bsf   PORTC,1                 ;/OE High (End Read)
                  bsf   PORTC,0                 ;/CE High / Deselect component (RAM)
         return
;__________________________________
;Write a Byte to Ext. RAM
;Byte to Write should be in W
;before calling this function.
Write_Byte_Ext_RAM:
                  movwf PORTB
                  bsf   PORTC,2                 ;/WE High
                  bcf   PORTC,0                 ;/CE Low / Select component (RAM)
                  bcf   PORTC,2                 ;/WE Low (Write)
                  bsf   PORTC,2                 ;/WE High (End Write)
                  bsf   PORTC,0                 ;/CE High / Deselect component (RAM)
         return
;__________________________________
;Get a Byte from serial Port
;Ensure Bank 0 is selected
;Byte received stored in W Register
Get_Byte          clrf  STATUS
                  btfss PIR1,5
                  goto  Get_Byte
                  movf  RCREG,w
         return
;__________________________________
;Get a Byte using serial Port
;Ensure Bank 0 is selected
;Byte send from W Register
Send_Byte:
                  clrf  STATUS
W_Put             bcf   STATUS,5
                  bcf   STATUS,6
                  btfss PIR1,4
                  goto  W_Put
                  movwf TXREG
         return
;__________________________________

Skip_Functions:

         goto $

         org 0x1800     ;PRGM. Memory Page 3
                        ;Displayed messages are stored from this address
         da "Game Boy Cartridge Filler/Dumper Prototype", 0x050D, 0x050D

         org 0x1820
         da "Ready>"

         org 0x1826
         da 0x050D, "Wrong Command", 0x050D, 0x050D

         org 0x1830
         da 0x050D, "Not Implemented", 0x050D, 0x050D

         org 0x1840
         da 0x050D, 0x050D
         da "CLR   Clear Screen", 0x050D
         da "DMP   Dump 32KB Cartridge to Serial Port", 0x050D
         da "FIL   Fill 32KB Cartridge with Serial Data", 0x050D
         da "HLP   Display the List of Available Commands", 0x050D
         da "RST   Reset", 0x050D, 0x050D

         org 0x18A0
         da 0x050D,"Ok", 0x050D, 0x050D

         org 0x18B0
         da 0x050D,"Cartridge Filled", 0x050D, 0x050D

         org 0x18C0
         da "Cartridge Dumped", 0x050D, 0x050D

         org 0x18D0
         da "Operation Cancelled", 0x050D, 0x050D

         ;Command Memory
         org 0x1FD0
         da "HGZ",0,0,0
         org 0x1FD8
         da "FIL",0,0,0
         org 0x1FE0
         da "DMP",0,0,0
         org 0x1FE8
         da "CLR",0,0,0
         org 0x1FF0
         da "RST",0,0,0
         org 0x1FF8
         da "HLP",0,0,0

         end

